# SPDX-FileCopyrightText: Red Hat, Inc.
# SPDX-License-Identifier: GPL-2.0-or-later

"""
This module provides DSA software versioning information for
python based components
"""
from __future__ import absolute_import
from __future__ import division

from vdsm import osinfo
from vdsm.common import cache


version_name = "Snow Man"
software_version = "4.50.6"
software_revision = "9.git989704532"
raw_version_revision = "4.50.6.9.git989704532"


def _get_libvirt_version():
    packages = osinfo.package_versions()
    libvirt_version = tuple(
        int(v)
        for v in packages['libvirt']['version'].split('.')
    )
    return libvirt_version[:2]


def _get_qemukvm_version():
    packages = osinfo.package_versions()
    qemukvm_version = tuple(
        int(v)
        for v in packages['qemu-kvm']['version'].split('.')
    )
    return qemukvm_version[:2]


@cache.memoized
def version_info():
    cluster_levels = ['4.2', '4.3', '4.4', '4.5', '4.6']
    libvirt_version = _get_libvirt_version()
    qemukvm_version = _get_qemukvm_version()

    if libvirt_version >= (8, 0):
        cluster_levels.append('4.7')
    if libvirt_version >= (9, 5) and qemukvm_version >= (8, 1):
        cluster_levels.append('4.8')

    return {
        'version_name': version_name,
        'software_version': software_version,
        'software_revision': software_revision,
        'supportedENGINEs': ['4.2', '4.3', '4.4', '4.5'],
        'clusterLevels': cluster_levels,
    }
